from ..testing import LINEAGE_CONTROLPANELS_INTEGRATION_TESTING
from collective.lineage.utils import enable_childsite
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.registry.interfaces import IRegistry
from Products.CMFPlone.interfaces import ISiteSchema
from zope.component import getUtility

import plone.api.content
import unittest

# Red pixel with filename pixel.png
LOGO_1_BASE64 = b"filenameb64:cGl4ZWwucG5n;datab64:iVBORw0KGgoAAAANSUhEUgAAAAEAAAABCAIAAACQd1PeAAAADElEQVQI12P4z8AAAAMBAQAY3Y2wAAAAAElFTkSuQmCC"  # noqa

# Green pixel with filename pixelgreen.png
LOGO_2_BASE64 = b"filenameb64:cGl4ZWxncmVlbi5wbmc=;datab64:iVBORw0KGgoAAAANSUhEUgAAAAEAAAABCAIAAACQd1PeAAAADElEQVQI12Ng+M8AAAICAQCqKp4nAAAAAElFTkSuQmCC"  # noqa


class SiteControlPanelFunctionalTest(unittest.TestCase):
    """Test that changes in the site control panel are actually
    stored in the registry.
    """

    layer = LINEAGE_CONTROLPANELS_INTEGRATION_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        self.portal_url = self.portal.absolute_url()

        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        plone.api.content.create(
            container=self.portal, type="Folder", id="subsite", title="subsite"
        )
        self.subsite = self.portal["subsite"]
        plone.api.content.transition(obj=self.subsite, transition="publish")
        enable_childsite(self.subsite)

    def _registry_edit(self, context, schema, data):
        """Edit data against the registry schema on a given context."""
        registry = getUtility(IRegistry, context=context)
        settings = registry.forInterface(schema, prefix="plone")
        for item in data:
            setattr(settings, item["name"], item["value"])

    def _registry_assert(self, context, schema, data):
        """Assert data against the registry schema on a given context."""
        registry = getUtility(IRegistry, context=context)
        settings = registry.forInterface(schema, prefix="plone")

        for item in data:
            self.assertEqual(getattr(settings, item["name"]), item["value"])

    def test_lineage_controlpanel_site_roundtrip(self):
        """Test editing the main site controlpanel and the lineage controlpanel
        in a subsite and check, if settings don't overwrite each other.
        """
        data_portal = [
            {"name": "site_title", "value": "Main portal"},
            {"name": "site_logo", "value": LOGO_1_BASE64},
            {"name": "exposeDCMetaTags", "value": True},
            {"name": "enable_sitemap", "value": True},
            {"name": "webstats_js", "value": "// dummy js"},
            {"name": "display_publication_date_in_byline", "value": True},
            {"name": "icon_visibility", "value": "enabled"},
            {"name": "thumb_visibility", "value": "enabled"},
            {"name": "toolbar_position", "value": "side"},
            {"name": "toolbar_logo", "value": "some_logo, nonexistent"},
            {"name": "robots_txt", "value": "please, no robots"},
            {"name": "default_page", "value": ["start.html"]},
            {"name": "roles_allowed_to_add_keywords", "value": ["Manager"]},
        ]

        data_subsite = [
            {"name": "site_title", "value": "Sub portal"},
            {"name": "site_logo", "value": LOGO_2_BASE64},
            {"name": "exposeDCMetaTags", "value": True},
            {"name": "enable_sitemap", "value": True},
            {"name": "webstats_js", "value": None},
            {"name": "display_publication_date_in_byline", "value": True},
            {"name": "icon_visibility", "value": "false"},
            {"name": "thumb_visibility", "value": "false"},
            {"name": "toolbar_position", "value": "side"},
            {"name": "toolbar_logo", "value": "another nonexistent logo"},
            {"name": "robots_txt", "value": "more robots!"},
            {"name": "default_page", "value": ["start.html"]},
            {"name": "roles_allowed_to_add_keywords", "value": ["Manager"]},
        ]

        # Populate main site
        self._registry_edit(self.portal, ISiteSchema, data_portal)

        # Populate child site
        self._registry_edit(self.subsite, ISiteSchema, data_subsite)

        # Assert main portal
        self._registry_assert(self.portal, ISiteSchema, data_portal)

        # Assert sub portal
        self._registry_assert(self.subsite, ISiteSchema, data_subsite)



from collective.lineage.interfaces import IChildSite
from Products.Five import BrowserView


class ActionAvailableView(BrowserView):
    def __call__(self):
        return IChildSite.providedBy(self.context)



from plone import api
from plone.app.testing import applyProfile
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import PloneSandboxLayer


class BrowserLayer(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        import collective.lineage

        self.loadZCML(package=collective.lineage)
        import lineage.registry

        self.loadZCML(package=lineage.registry)
        import lineage.controlpanels

        self.loadZCML(package=lineage.controlpanels)

    def setUpPloneSite(self, portal):
        applyProfile(portal, "lineage.controlpanels:default")

        wf_tool = api.portal.get_tool("portal_workflow")
        wf_tool.setDefaultChain("simple_publication_workflow")


LINEAGE_CONTROLPANELS_FIXTURE = BrowserLayer()


LINEAGE_CONTROLPANELS_INTEGRATION_TESTING = IntegrationTesting(
    bases=(LINEAGE_CONTROLPANELS_FIXTURE,), name="BrowserLayer:IntegrationTesting"
)


LINEAGE_CONTROLPANELS_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(LINEAGE_CONTROLPANELS_FIXTURE,), name="BrowserLayer:FunctionalTesting"
)



"""Installer for the lineage.controlpanels package."""

from setuptools import setup

version = "2.0.0"
short_description = "Plone controlpanels for Lineage sites"
long_description = "\n\n".join(
    [
        open("README.rst").read(),
        open("CONTRIBUTORS.rst").read(),
        open("CHANGES.rst").read(),
    ]
)


setup(
    name="lineage.controlpanels",
    version=version,
    description=short_description,
    long_description=long_description,
    # Get more from https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Environment :: Web Environment",
        "Framework :: Plone",
        "Framework :: Plone :: Addon",
        "Framework :: Plone :: 6.1",
        "Framework :: Plone :: 6.2",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Programming Language :: Python :: 3.13",
        "Programming Language :: Python :: 3.14",
        "Operating System :: OS Independent",
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
    ],
    keywords="Python Plone",
    author="Johannes Raggam",
    author_email="thetetet@gmail.com",
    url="https://pypi.python.org/pypi/lineage.controlpanels",
    license="GPL version 2",
    include_package_data=True,
    zip_safe=False,
    python_requires=">=3.10",
    install_requires=[
        "setuptools",
        # -*- Extra requirements: -*-
        "collective.lineage>=2.1",
        "lineage.registry>=1.3.3",
    ],
    extras_require={
        "test": [
            "plone.app.testing",
            "plone.api",
        ],
    },
    entry_points="""
    [z3c.autoinclude.plugin]
    target = plone
    """,
)
